@@ -107,6 +107,15 @@ static INLINE void load_u8_8x4(const uint8_t *s, const ptrdiff_t p, uint8x8_t *c
     *s3 = vld1_u8(s);
 }
 
+static INLINE void load_u8_8x3(const uint8_t *s, const ptrdiff_t p, uint8x8_t *const s0, uint8x8_t *const s1,
+                               uint8x8_t *const s2) {
+    *s0 = vld1_u8(s);
+    s += p;
+    *s1 = vld1_u8(s);
+    s += p;
+    *s2 = vld1_u8(s);
+}
+
 static INLINE void load_u16_4x4(const uint16_t *s, const ptrdiff_t p, uint16x4_t *const s0, uint16x4_t *const s1,
                                 uint16x4_t *const s2, uint16x4_t *const s3) {
     *s0 = vld1_u16(s);
@@ -119,6 +128,22 @@ static INLINE void load_u16_4x4(const uint16_t *s, const ptrdiff_t p, uint16x4_t
     s += p;
 }
 
+static INLINE void load_u16_4x6(const uint16_t *s, ptrdiff_t p, uint16x4_t *const s0, uint16x4_t *const s1,
+                                uint16x4_t *const s2, uint16x4_t *const s3, uint16x4_t *const s4,
+                                uint16x4_t *const s5) {
+    *s0 = vld1_u16(s);
+    s += p;
+    *s1 = vld1_u16(s);
+    s += p;
+    *s2 = vld1_u16(s);
+    s += p;
+    *s3 = vld1_u16(s);
+    s += p;
+    *s4 = vld1_u16(s);
+    s += p;
+    *s5 = vld1_u16(s);
+}
+
 static INLINE void load_u16_4x7(const uint16_t *s, ptrdiff_t p, uint16x4_t *const s0, uint16x4_t *const s1,
                                 uint16x4_t *const s2, uint16x4_t *const s3, uint16x4_t *const s4, uint16x4_t *const s5,
                                 uint16x4_t *const s6) {
@@ -137,6 +162,60 @@ static INLINE void load_u16_4x7(const uint16_t *s, ptrdiff_t p, uint16x4_t *cons
     *s6 = vld1_u16(s);
 }
 
+static INLINE void load_u16_4x8(const uint16_t *s, ptrdiff_t p, uint16x4_t *const s0, uint16x4_t *const s1,
+                                uint16x4_t *const s2, uint16x4_t *const s3, uint16x4_t *const s4, uint16x4_t *const s5,
+                                uint16x4_t *const s6, uint16x4_t *const s7) {
+    *s0 = vld1_u16(s);
+    s += p;
+    *s1 = vld1_u16(s);
+    s += p;
+    *s2 = vld1_u16(s);
+    s += p;
+    *s3 = vld1_u16(s);
+    s += p;
+    *s4 = vld1_u16(s);
+    s += p;
+    *s5 = vld1_u16(s);
+    s += p;
+    *s6 = vld1_u16(s);
+    s += p;
+    *s7 = vld1_u16(s);
+}
+
+static INLINE void load_u16_4x14(const uint16_t *s, ptrdiff_t p, uint16x4_t *const s0, uint16x4_t *const s1,
+                                 uint16x4_t *const s2, uint16x4_t *const s3, uint16x4_t *const s4, uint16x4_t *const s5,
+                                 uint16x4_t *const s6, uint16x4_t *const s7, uint16x4_t *const s8, uint16x4_t *const s9,
+                                 uint16x4_t *const s10, uint16x4_t *const s11, uint16x4_t *const s12,
+                                 uint16x4_t *const s13) {
+    *s0 = vld1_u16(s);
+    s += p;
+    *s1 = vld1_u16(s);
+    s += p;
+    *s2 = vld1_u16(s);
+    s += p;
+    *s3 = vld1_u16(s);
+    s += p;
+    *s4 = vld1_u16(s);
+    s += p;
+    *s5 = vld1_u16(s);
+    s += p;
+    *s6 = vld1_u16(s);
+    s += p;
+    *s7 = vld1_u16(s);
+    s += p;
+    *s8 = vld1_u16(s);
+    s += p;
+    *s9 = vld1_u16(s);
+    s += p;
+    *s10 = vld1_u16(s);
+    s += p;
+    *s11 = vld1_u16(s);
+    s += p;
+    *s12 = vld1_u16(s);
+    s += p;
+    *s13 = vld1_u16(s);
+}
+
 static INLINE void load_s16_8x2(const int16_t *s, const ptrdiff_t p, int16x8_t *const s0, int16x8_t *const s1) {
     *s0 = vld1q_s16(s);
     s += p;
@@ -310,6 +389,15 @@ static INLINE void load_s16_4x5(const int16_t *s, ptrdiff_t p, int16x4_t *const
     *s4 = vld1_s16(s);
 }
 
+static INLINE void load_s16_4x3(const int16_t *s, ptrdiff_t p, int16x4_t *const s0, int16x4_t *const s1,
+                                int16x4_t *const s2) {
+    *s0 = vld1_s16(s);
+    s += p;
+    *s1 = vld1_s16(s);
+    s += p;
+    *s2 = vld1_s16(s);
+}
+
 static INLINE void load_u16_4x5(const uint16_t *s, const ptrdiff_t p, uint16x4_t *const s0, uint16x4_t *const s1,
                                 uint16x4_t *const s2, uint16x4_t *const s3, uint16x4_t *const s4) {
     *s0 = vld1_u16(s);
@@ -432,6 +520,50 @@ static INLINE void store_u16_8x8(uint16_t *s, ptrdiff_t dst_stride, const uint16
     vst1q_u16(s, s7);
 }
 
+static INLINE void store_u16_4x12(uint16_t *s, ptrdiff_t dst_stride, const uint16x4_t s0, const uint16x4_t s1,
+                                  const uint16x4_t s2, const uint16x4_t s3, const uint16x4_t s4, const uint16x4_t s5,
+                                  const uint16x4_t s6, const uint16x4_t s7, const uint16x4_t s8, const uint16x4_t s9,
+                                  const uint16x4_t s10, const uint16x4_t s11) {
+    vst1_u16(s, s0);
+    s += dst_stride;
+    vst1_u16(s, s1);
+    s += dst_stride;
+    vst1_u16(s, s2);
+    s += dst_stride;
+    vst1_u16(s, s3);
+    s += dst_stride;
+    vst1_u16(s, s4);
+    s += dst_stride;
+    vst1_u16(s, s5);
+    s += dst_stride;
+    vst1_u16(s, s6);
+    s += dst_stride;
+    vst1_u16(s, s7);
+    s += dst_stride;
+    vst1_u16(s, s8);
+    s += dst_stride;
+    vst1_u16(s, s9);
+    s += dst_stride;
+    vst1_u16(s, s10);
+    s += dst_stride;
+    vst1_u16(s, s11);
+}
+
+static INLINE void store_u16_4x6(uint16_t *s, ptrdiff_t dst_stride, const uint16x4_t s0, const uint16x4_t s1,
+                                 const uint16x4_t s2, const uint16x4_t s3, const uint16x4_t s4, const uint16x4_t s5) {
+    vst1_u16(s, s0);
+    s += dst_stride;
+    vst1_u16(s, s1);
+    s += dst_stride;
+    vst1_u16(s, s2);
+    s += dst_stride;
+    vst1_u16(s, s3);
+    s += dst_stride;
+    vst1_u16(s, s4);
+    s += dst_stride;
+    vst1_u16(s, s5);
+}
+
 static INLINE void store_u16_4x4(uint16_t *s, ptrdiff_t dst_stride, const uint16x4_t s0, const uint16x4_t s1,
                                  const uint16x4_t s2, const uint16x4_t s3) {
     vst1_u16(s, s0);
@@ -443,12 +575,30 @@ static INLINE void store_u16_4x4(uint16_t *s, ptrdiff_t dst_stride, const uint16
     vst1_u16(s, s3);
 }
 
+static INLINE void store_u16_4x3(uint16_t *s, ptrdiff_t dst_stride, const uint16x4_t s0, const uint16x4_t s1,
+                                 const uint16x4_t s2) {
+    vst1_u16(s, s0);
+    s += dst_stride;
+    vst1_u16(s, s1);
+    s += dst_stride;
+    vst1_u16(s, s2);
+}
+
 static INLINE void store_u16_8x2(uint16_t *s, ptrdiff_t dst_stride, const uint16x8_t s0, const uint16x8_t s1) {
     vst1q_u16(s, s0);
     s += dst_stride;
     vst1q_u16(s, s1);
 }
 
+static INLINE void store_u16_8x3(uint16_t *s, ptrdiff_t dst_stride, const uint16x8_t s0, const uint16x8_t s1,
+                                 const uint16x8_t s2) {
+    vst1q_u16(s, s0);
+    s += dst_stride;
+    vst1q_u16(s, s1);
+    s += dst_stride;
+    vst1q_u16(s, s2);
+}
+
 static INLINE void store_u16_8x4(uint16_t *s, ptrdiff_t dst_stride, const uint16x8_t s0, const uint16x8_t s1,
                                  const uint16x8_t s2, const uint16x8_t s3) {
     vst1q_u16(s, s0);
@@ -480,6 +630,26 @@ static INLINE void store_s16_8x8(int16_t *s, ptrdiff_t dst_stride, const int16x8
     vst1q_s16(s, s7);
 }
 
+static INLINE void store_s16_4x8(int16_t *s, ptrdiff_t dst_stride, const int16x4_t s0, const int16x4_t s1,
+                                 const int16x4_t s2, const int16x4_t s3, const int16x4_t s4, const int16x4_t s5,
+                                 const int16x4_t s6, const int16x4_t s7) {
+    vst1_s16(s, s0);
+    s += dst_stride;
+    vst1_s16(s, s1);
+    s += dst_stride;
+    vst1_s16(s, s2);
+    s += dst_stride;
+    vst1_s16(s, s3);
+    s += dst_stride;
+    vst1_s16(s, s4);
+    s += dst_stride;
+    vst1_s16(s, s5);
+    s += dst_stride;
+    vst1_s16(s, s6);
+    s += dst_stride;
+    vst1_s16(s, s7);
+}
+
 static INLINE void store_s16_4x4(int16_t *s, ptrdiff_t dst_stride, const int16x4_t s0, const int16x4_t s1,
                                  const int16x4_t s2, const int16x4_t s3) {
     vst1_s16(s, s0);
@@ -491,14 +661,38 @@ static INLINE void store_s16_4x4(int16_t *s, ptrdiff_t dst_stride, const int16x4
     vst1_s16(s, s3);
 }
 
-/* These intrinsics require immediate values, so we must use #defines
-   to enforce that. */
-#define store_s16_2x1(s, s0, lane) \
-    do { vst1_lane_s32((int32_t *)(s), vreinterpret_s32_s16(s0), lane); } while (0)
-#define store_u16_2x1(s, s0, lane) \
-    do { vst1_lane_u32((uint32_t *)(s), vreinterpret_u32_u16(s0), lane); } while (0)
-#define store_u16q_2x1(s, s0, lane) \
-    do { vst1q_lane_u32((uint32_t *)(s), vreinterpretq_u32_u16(s0), lane); } while (0)
+static INLINE void store_s16_8x2(int16_t *s, ptrdiff_t dst_stride, const int16x8_t s0, const int16x8_t s1) {
+    vst1q_s16(s, s0);
+    s += dst_stride;
+    vst1q_s16(s, s1);
+}
+
+#define store_u16_2x1_lane(dst, src, lane)                           \
+    do {                                                             \
+        uint32_t a = vget_lane_u32(vreinterpret_u32_u16(src), lane); \
+        memcpy(dst, &a, 4);                                          \
+    } while (0)
+
+#define store_s16_2x1_lane(dst, src, lane)                          \
+    do {                                                            \
+        int32_t a = vget_lane_s32(vreinterpret_s32_s16(src), lane); \
+        memcpy(dst, &a, 4);                                         \
+    } while (0)
+
+#define store_u16_4x1_lane(dst, src, lane)                             \
+    do {                                                               \
+        uint64_t a = vgetq_lane_u64(vreinterpretq_u64_u16(src), lane); \
+        memcpy(dst, &a, 8);                                            \
+    } while (0)
+
+#define store_s16_4x1_lane(dst, src, lane)                            \
+    do {                                                              \
+        int64_t a = vgetq_lane_s64(vreinterpretq_s64_s16(src), lane); \
+        memcpy(dst, &a, 8);                                           \
+    } while (0)
+
+// Store the low 32-bits from a single vector.
+static INLINE void store_u16_2x1(uint16_t *dst, const uint16x4_t src) { store_u16_2x1_lane(dst, src, 0); }
 
 static INLINE void store_s16_8x4(int16_t *s, ptrdiff_t dst_stride, const int16x8_t s0, const int16x8_t s1,
                                  const int16x8_t s2, const int16x8_t s3) {
@@ -740,6 +934,21 @@ static INLINE void load_s16_8x4(const int16_t *s, ptrdiff_t p, int16x8_t *const
     *s3 = vld1q_s16(s);
 }
 
+static INLINE void load_s16_8x3(const int16_t *s, ptrdiff_t p, int16x8_t *const s0, int16x8_t *const s1,
+                                int16x8_t *const s2) {
+    *s0 = vld1q_s16(s);
+    s += p;
+    *s1 = vld1q_s16(s);
+    s += p;
+    *s2 = vld1q_s16(s);
+}
+
+#define load_unaligned_u32_2x1_lane(v, p, lane) \
+    do { (v) = vld1_lane_u32((const uint32_t *)(p), (v), (lane)); } while (0)
+
+#define load_unaligned_u32_4x1_lane(v, p, lane) \
+    do { (v) = vld1q_lane_u32((const uint32_t *)(p), (v), (lane)); } while (0)
+
 // Load 2 sets of 4 bytes when alignment is not guaranteed.
 static INLINE uint8x8_t load_unaligned_u8(const uint8_t *buf, int stride) {
     uint32_t a;
@@ -893,6 +1102,15 @@ static INLINE void load_u8_16x8(const uint8_t *s, ptrdiff_t p, uint8x16_t *const
     *s7 = vld1q_u8(s);
 }
 
+static INLINE void load_u8_16x3(const uint8_t *s, ptrdiff_t p, uint8x16_t *const s0, uint8x16_t *const s1,
+                                uint8x16_t *const s2) {
+    *s0 = vld1q_u8(s);
+    s += p;
+    *s1 = vld1q_u8(s);
+    s += p;
+    *s2 = vld1q_u8(s);
+}
+
 static INLINE void load_u8_16x4(const uint8_t *s, ptrdiff_t p, uint8x16_t *const s0, uint8x16_t *const s1,
                                 uint8x16_t *const s2, uint8x16_t *const s3) {
     *s0 = vld1q_u8(s);
@@ -904,6 +1122,19 @@ static INLINE void load_u8_16x4(const uint8_t *s, ptrdiff_t p, uint8x16_t *const
     *s3 = vld1q_u8(s);
 }
 
+static INLINE void load_u8_16x5(const uint8_t *s, ptrdiff_t p, uint8x16_t *const s0, uint8x16_t *const s1,
+                                uint8x16_t *const s2, uint8x16_t *const s3, uint8x16_t *const s4) {
+    *s0 = vld1q_u8(s);
+    s += p;
+    *s1 = vld1q_u8(s);
+    s += p;
+    *s2 = vld1q_u8(s);
+    s += p;
+    *s3 = vld1q_u8(s);
+    s += p;
+    *s4 = vld1q_u8(s);
+}
+
 static INLINE void load_u16_8x8(const uint16_t *s, const ptrdiff_t p, uint16x8_t *s0, uint16x8_t *s1, uint16x8_t *s2,
                                 uint16x8_t *s3, uint16x8_t *s4, uint16x8_t *s5, uint16x8_t *s6, uint16x8_t *s7) {
     *s0 = vld1q_u16(s);
@@ -973,6 +1204,20 @@ static INLINE uint16x8_t load_unaligned_u16_4x2(const uint16_t *buf, uint32_t st
     return vreinterpretq_u16_u64(a_u64);
 }
 
+static INLINE int16x8_t load_unaligned_s16_4x2(const int16_t *buf, uint32_t stride) {
+    int64_t   a;
+    int64x2_t a_s64;
+
+    memcpy(&a, buf, 8);
+    buf += stride;
+    a_s64 = vdupq_n_s64(0);
+    a_s64 = vsetq_lane_s64(a, a_s64, 0);
+    memcpy(&a, buf, 8);
+    buf += stride;
+    a_s64 = vsetq_lane_s64(a, a_s64, 1);
+    return vreinterpretq_s16_s64(a_s64);
+}
+
 static INLINE void load_unaligned_u16_4x4(const uint16_t *buf, uint32_t stride, uint16x8_t *tu0, uint16x8_t *tu1) {
     *tu0 = load_unaligned_u16_4x2(buf, stride);
     buf += 2 * stride;
@@ -1020,6 +1265,25 @@ static INLINE void store_u32_4x4(uint32_t *s, int32_t p, uint32x4_t s1, uint32x4
     vst1q_u32(s, s4);
 }
 
+static INLINE void store_s32_8x4(int32_t *s, int32_t p, int32x4_t s0, int32x4_t s1, int32x4_t s2, int32x4_t s3,
+                                 int32x4_t s4, int32x4_t s5, int32x4_t s6, int32x4_t s7) {
+    vst1q_s32(s, s0);
+    s += p;
+    vst1q_s32(s, s1);
+    s += p;
+    vst1q_s32(s, s2);
+    s += p;
+    vst1q_s32(s, s3);
+    s += p;
+    vst1q_s32(s, s4);
+    s += p;
+    vst1q_s32(s, s5);
+    s += p;
+    vst1q_s32(s, s6);
+    s += p;
+    vst1q_s32(s, s7);
+}
+
 static INLINE int16x8_t load_tran_low_to_s16q(const tran_low_t *buf) {
     const int32x4_t v0 = vld1q_s32(buf);
     const int32x4_t v1 = vld1q_s32(buf + 4);
@@ -1064,4 +1328,67 @@ static INLINE void store_unaligned_u16_4x2(uint16_t *dst, uint32_t dst_stride, u
     store_unaligned_u16_4x1(dst, src, 1);
 }
 
+// The `lane` parameter here must be an immediate.
+#define store_u8_2x1_lane(dst, src, lane)                           \
+    do {                                                            \
+        uint16_t a = vget_lane_u16(vreinterpret_u16_u8(src), lane); \
+        memcpy(dst, &a, 2);                                         \
+    } while (0)
+
+#define store_u8_4x1_lane(dst, src, lane)                           \
+    do {                                                            \
+        uint32_t a = vget_lane_u32(vreinterpret_u32_u8(src), lane); \
+        memcpy(dst, &a, 4);                                         \
+    } while (0)
+
+// Store two blocks of 32-bits from a single vector.
+static INLINE void store_u8x4_strided_x2(uint8_t *dst, ptrdiff_t stride, uint8x8_t src) {
+    store_u8_4x1_lane(dst, src, 0);
+    dst += stride;
+    store_u8_4x1_lane(dst, src, 1);
+}
+
+// Store two blocks of 16-bits from a single vector.
+static inline void store_u8x2_strided_x2(uint8_t *dst, uint32_t dst_stride, uint8x8_t src) {
+    store_u8_2x1_lane(dst, src, 0);
+    dst += dst_stride;
+    store_u8_2x1_lane(dst, src, 1);
+}
+
+#undef store_u8_4x1_lane
+#undef store_u8_2x1_lane
+
+// Store two blocks of 64-bits from a single vector.
+static INLINE void store_s16x4_strided_x2(int16_t *dst, int32_t dst_stride, int16x8_t src) {
+    store_s16_4x1_lane(dst, src, 0);
+    dst += dst_stride;
+    store_s16_4x1_lane(dst, src, 1);
+}
+
+// Store two blocks of 32-bits from a single vector.
+static inline void store_u16x2_strided_x2(uint16_t *dst, uint32_t dst_stride, uint16x4_t src) {
+    store_u16_2x1_lane(dst, src, 0);
+    dst += dst_stride;
+    store_u16_2x1_lane(dst, src, 1);
+}
+
+// Store two blocks of 32-bits from a single vector.
+static inline void store_s16x2_strided_x2(int16_t *dst, uint32_t dst_stride, int16x4_t src) {
+    store_s16_2x1_lane(dst, src, 0);
+    dst += dst_stride;
+    store_s16_2x1_lane(dst, src, 1);
+}
+
+// Store two blocks of 64-bits from a single vector.
+static INLINE void store_u16x4_strided_x2(uint16_t *dst, uint32_t dst_stride, uint16x8_t src) {
+    store_u16_4x1_lane(dst, src, 0);
+    dst += dst_stride;
+    store_u16_4x1_lane(dst, src, 1);
+}
+
+#undef store_u16_2x1_lane
+#undef store_s16_2x1_lane
+#undef store_u16_4x1_lane
+#undef store_s16_4x1_lane
+
 #endif // AOM_AOM_DSP_ARM_MEM_NEON_H_
