@@ -38,11 +38,11 @@ extern "C" {
  * Macros
  **************************************/
 
-#define GROUP_OF_4_8x8_BLOCKS(org_x, org_y) (((org_x >> 3) & 0x1) && ((org_y >> 3) & 0x1) ? TRUE : FALSE)
+#define GROUP_OF_4_8x8_BLOCKS(org_x, org_y) (((org_x >> 3) & 0x1) && ((org_y >> 3) & 0x1) ? true : false)
 #define GROUP_OF_4_16x16_BLOCKS(org_x, org_y) \
-    (((((org_x >> 3) & 0x2) == 0x2) && (((org_y >> 3) & 0x2) == 0x2)) ? TRUE : FALSE)
+    (((((org_x >> 3) & 0x2) == 0x2) && (((org_y >> 3) & 0x2) == 0x2)) ? true : false)
 #define GROUP_OF_4_32x32_BLOCKS(org_x, org_y) \
-    (((((org_x >> 3) & 0x4) == 0x4) && (((org_y >> 3) & 0x4) == 0x4)) ? TRUE : FALSE)
+    (((((org_x >> 3) & 0x4) == 0x4) && (((org_y >> 3) & 0x4) == 0x4)) ? true : false)
 
 /**************************************
  * Coding Loop Context
@@ -107,8 +107,6 @@ typedef struct InterCompCtrls {
     uint8_t pred0_to_pred1_mult;
     // if true, use rate @ compound params derivation
     uint8_t use_rate;
-    //no compound for low complexity blocks (MVP only)
-    uint8_t mvp_no_cmp_low_cmplx;
     //no diff for nsq (MVP only)
     uint8_t mvp_no_diff_nsq;
     //no wedge if blk variance is less than mvp_no_wdg_var_th; 0: OFF (MVP only)
@@ -412,9 +410,11 @@ typedef struct MdSubPelSearchCtrls {
     // Skip high precision (1/8-Pel) when the ME vs. MVP MV difference (x or y) is larger than the threshold.
     // MAX_SIGNED_VALUE is off; 0 is safest on TH, higher is more aggressive
     int hp_mv_th;
+    int32_t
+        bias_fp; // Bias towards fpel at the MD subpel-search: apply a penalty to the cost of fractional positions during the subpel-search each time we check against a full-pel MV
 } MdSubPelSearchCtrls;
 typedef struct ParentSqCmplxCtrls {
-    Bool enabled;
+    bool enabled;
 
     // cutoff for the highest coeff-area band [0-100]
     uint8_t high_freq_band1_th;
@@ -536,7 +536,7 @@ typedef struct NicPruningCtrls {
     uint64_t mds3_cand_base_th;
 
     // enable skipping MDS1 in PD1 when there is only 1 cand post-mds0
-    Bool enable_skipping_mds1;
+    bool enable_skipping_mds1;
     // if (best_mds0_distortion/QP < TH) consider only the best candidate after MDS0; 0: OFF,
     // higher: more aggressive.
     uint32_t force_1_cand_th;
@@ -662,7 +662,7 @@ typedef struct WmCtrls {
     // Skip if alpha/ beta / gamma / delta is higher than threshold value
     uint16_t upper_band_th;
     // Shut the approximation(s) if refinement @ mds1 or mds3
-    Bool shut_approx_if_not_mds0;
+    bool shut_approx_if_not_mds0;
 } WmCtrls;
 typedef struct UvCtrls {
     uint8_t enabled;
@@ -698,7 +698,7 @@ typedef struct InterpolationSearchCtrls {
 } InterpolationSearchCtrls;
 typedef struct SpatialSSECtrls {
     // enable spatial-sse for each superblock
-    Bool spatial_sse_full_loop_level;
+    bool spatial_sse_full_loop_level;
 } SpatialSSECtrls;
 typedef struct RedundantCandCtrls {
     int score_th;
@@ -722,7 +722,7 @@ typedef struct Lpd0Ctrls {
     // Whether light-PD0 is set to be used for an SB (the detector may change this)
     Pd0Level pd0_level;
     // Whether to use a detector; if use_light_pd0 is set to 1, the detector will protect tough SBs
-    Bool use_lpd0_detector[LPD0_LEVELS];
+    bool use_lpd0_detector[LPD0_LEVELS];
     // Use info of ref frames - incl. colocated SBs - such as mode, coeffs, etc. in the detector.
     // [0,2] - 0 is off, 2 is most aggressive
     uint8_t use_ref_info[LPD0_LEVELS];
@@ -738,7 +738,7 @@ typedef struct Lpd1Ctrls {
     // Whether light-PD1 is set to be used for an SB (the detector may change this)
     Pd1Level pd1_level;
     // Whether to use a detector; if use_light_pd1 is set to 1, the detector will protect tough SBs
-    Bool use_lpd1_detector[LPD1_LEVELS];
+    bool use_lpd1_detector[LPD1_LEVELS];
     // Use info of ref frames - incl. colocated SBs - such as mode, coeffs, etc. in the detector.
     // [0,1] - 0 is off, 1 is on
     uint8_t use_ref_info[LPD1_LEVELS];
@@ -768,8 +768,6 @@ typedef struct DetectHighFreqCtrls {
     // me-SAD-to-SATD deviation of the 32x32 blocks threshold beyond which the SB is tagged (~2x is
     // the fundamental deviation)
     uint16_t satd_to_sad_dev_th;
-    // depth-removal level left-shift for the detected SB(s)
-    uint8_t depth_removal_shift;
     // maximum lpd0 level for the detected SB(s)
     uint8_t max_pic_lpd0_lvl;
     // maximum lpd1 level for the detected SB(s)
@@ -798,16 +796,16 @@ typedef struct Lpd1TxCtrls {
     uint8_t use_uv_shortcuts_on_y_coeffs;
 } Lpd1TxCtrls;
 typedef struct CflCtrls {
-    Bool enabled;
+    bool enabled;
     // Early exit to reduce the number of iterations to compute CFL parameters
     uint8_t itr_th;
 } CflCtrls;
 typedef struct MdRateEstCtrls {
     // If true, update skip context and dc_sign context (updates are done in the same func, so
     // control together)
-    Bool update_skip_ctx_dc_sign_ctx;
+    bool update_skip_ctx_dc_sign_ctx;
     // If true, update skip coeff context
-    Bool update_skip_coeff_ctx;
+    bool update_skip_coeff_ctx;
     // 0: OFF (always use approx for coeff rate), 1: full; always compute coeff rate, 2: when
     // num_coeff is low, use approximation for coeff rate
     uint8_t coeff_rate_est_lvl;
@@ -842,12 +840,11 @@ typedef struct TxShortcutCtrls {
 typedef struct Mds0Ctrls {
     // Distortion metric to use MDS0: SSD, VAR, SAD
     uint8_t mds0_dist_type;
-    // Skip cost computation if distortion is mds0_distortion_th % higher than best candidate cost
-    // (applies to reg. PD1 only)
-    uint8_t enable_cost_based_early_exit;
-    // % TH used to compare candidate distortion to best cost; higher is safer (applies to reg. PD1
-    // only)
-    uint16_t mds0_distortion_th;
+    // 0: disabled, > 0: switch between: (1) reset reference cost for each subsequent class, (2) continuously update reference cost, (uint8_t) ~0: continuously update reference cost
+    uint8_t pruning_method_th;
+    // % TH(s) used to compare candidate distortion to best cost; higher is safer (applies to reg. PD1 only)
+    uint16_t per_class_dist_to_cost_th[CAND_CLASS_TOTAL];
+    uint16_t dist_to_cost_th;
 } Mds0Ctrls;
 typedef struct CandReductionCtrls {
     RedundantCandCtrls redundant_cand_ctrls;
@@ -870,6 +867,12 @@ typedef struct SkipSubDepthCtrls {
     uint8_t coeff_perc;
 
 } SkipSubDepthCtrls;
+typedef struct FilterIntraCtrls {
+    bool enabled;
+    // Set the max filter intra mode to test. The max filter intra level will also depend on ctx->intra_ctrls.intra_mode_end.
+    // The max mode to be test will be min(max_filter_intra_mode, ctx->intra_ctrls.intra_mode_end).
+    FilterIntraMode max_filter_intra_mode;
+} FilterIntraCtrls;
 
 typedef struct CompoundPredictionStore {
     //avoid doing Unipred prediction for redundant MV
@@ -938,7 +941,7 @@ typedef struct ModeDecisionContext {
     // for the case of lambda modulation (blk_lambda_tuning), full_lambda_md/fast_lambda_md
     // corresponds to block lambda and full_sb_lambda_md is the full lambda per sb
     uint32_t full_sb_lambda_md[2];
-    Bool     blk_lambda_tuning;
+    bool     blk_lambda_tuning;
     // Context Variables---------------------------------
     SuperBlock      *sb_ptr;
     BlkStruct       *blk_ptr;
@@ -963,8 +966,8 @@ typedef struct ModeDecisionContext {
     uint8_t          me_q_index;
     uint64_t         three_quad_energy;
     uint32_t         txb_1d_offset;
-    Bool             uv_intra_comp_only;
-    Bool             ind_uv_avail; // True if independent chroma search data is available
+    bool             uv_intra_comp_only;
+    bool             ind_uv_avail; // True if independent chroma search data is available
     UvPredictionMode best_uv_mode[UV_PAETH_PRED + 1];
     int8_t           best_uv_angle[UV_PAETH_PRED + 1];
     uint64_t         best_uv_cost[UV_PAETH_PRED + 1];
@@ -1008,7 +1011,7 @@ typedef struct ModeDecisionContext {
     uint8_t           nic_level;
     uint8_t          *cfl_temp_luma_recon;
     uint16_t         *cfl_temp_luma_recon16bit;
-    Bool              blk_skip_decision;
+    bool              blk_skip_decision;
     int8_t            rdoq_level;
     int16_t           sb_me_mv[MAX_NUM_OF_REF_PIC_LIST][MAX_REF_IDX][2];
     // Store ME MV of the square to use with NSQ shapes; 4x4 will also use the 8x8 ME MVs
@@ -1050,18 +1053,18 @@ typedef struct ModeDecisionContext {
     uint8_t   lpd1_allow_skipping_tx;
     // fast_loop_core signals
     // was md_staging_skip_interpolation_search
-    Bool mds_skip_ifs;
-    Bool mds_skip_uv_pred;
+    bool mds_skip_ifs;
+    bool mds_skip_uv_pred;
     // full_loop_core signals 0: perform luma & chroma prediction + interpolation search, 2: nothing
     // (use information from previous stages)
-    Bool mds_do_inter_pred;
+    bool mds_do_inter_pred;
     // 0: Tx Size recon only, 1:Tx Size search and recon
-    Bool mds_tx_size_mode;
-    Bool mds_txt_level;
-    Bool mds_skip_full_uv;
-    Bool mds_skip_rdoq;
-    Bool mds_spatial_sse;
-    Bool mds_do_intra_uv_pred;
+    bool mds_tx_size_mode;
+    bool mds_txt_level;
+    bool mds_skip_full_uv;
+    bool mds_skip_rdoq;
+    bool mds_spatial_sse;
+    bool mds_do_intra_uv_pred;
     // Store intra prediction for inter-intra
     uint8_t **intrapred_buf;
     // Store OBMC pre-computed data
@@ -1080,7 +1083,7 @@ typedef struct ModeDecisionContext {
     // Control subres_step per mds
     uint8_t           mds_subres_step;
     uint8_t           md_pic_obmc_level;
-    uint8_t           md_filter_intra_level;
+    FilterIntraCtrls  filter_intra_ctrls;
     uint8_t           md_allow_intrabc;
     uint8_t           md_palette_level;
     uint8_t           dist_based_ref_pruning;
@@ -1150,6 +1153,7 @@ typedef struct ModeDecisionContext {
     NicCtrls        nic_ctrls;
     MV              ref_mv;
     uint16_t        sb_index;
+    uint64_t        mds0_best_cost_per_class[CAND_CLASS_TOTAL];
     uint64_t        mds0_best_cost;
     uint8_t         mds0_best_class;
     uint32_t        mds0_best_idx;
@@ -1171,9 +1175,9 @@ typedef struct ModeDecisionContext {
     uint32_t                 max_nics_uv;
     InterpolationSearchCtrls ifs_ctrls;
     // If enabled, will bypass EncDec and copy recon/quant coeffs from MD
-    Bool bypass_encdec;
+    bool bypass_encdec;
     // Indicates whether only pred depth refinement is used in PD1 (set per SB)
-    Bool pred_depth_only;
+    bool pred_depth_only;
     // If true, indicates that there is a fixed partition structure for the current PD pass. When
     // the partition strucutre is fixed, there is no SQ/NSQ (d1) decision and no inter-depth (d2) decision.
     // When the partition structure in PD1 is fixed and EncDec is bypassed, the recon pic and QP info
@@ -1181,7 +1185,7 @@ typedef struct ModeDecisionContext {
     bool fixed_partition;
     // Indicates whether only pred depth refinement is used in PD1 (set per frame) Per frame is
     // necessary because some shortcuts can only be taken if the whole frame uses pred depth only
-    Bool                pic_pred_depth_only;
+    bool                pic_pred_depth_only;
     uint16_t            coded_area_sb;
     uint16_t            coded_area_sb_uv;
     Lpd0Ctrls           lpd0_ctrls;
@@ -1226,11 +1230,13 @@ typedef struct ModeDecisionContext {
     uint8_t lpd1_bypass_tx_th_div;
     // chroma components to compensate at MDS3 of LPD1
     COMPONENT_TYPE lpd1_chroma_comp;
+    uint8_t        lpd1_shift_mds0_dist;
     uint8_t        corrupted_mv_check;
     uint8_t        pred_mode_depth_refine;
     // when MD is done on 8bit, scale palette colors to 10bit (valid when bypass is 1)
     uint8_t  scale_palette;
     uint8_t  high_freq_present;
+    uint32_t high_freq_satd_to_me;
     uint32_t b32_satd[4];
     uint64_t rec_dist_per_quadrant[4];
     // non-normative txs
@@ -1245,17 +1251,15 @@ typedef struct ModeDecisionContext {
 } ModeDecisionContext;
 
 typedef void (*EbAv1LambdaAssignFunc)(PictureControlSet *pcs, uint32_t *fast_lambda, uint32_t *full_lambda,
-                                      uint8_t bit_depth, uint16_t qp_index, Bool multiply_lambda);
+                                      uint8_t bit_depth, uint16_t qp_index, bool multiply_lambda);
 
 /**************************************
  * Extern Function Declarations
  **************************************/
-extern EbErrorType svt_aom_mode_decision_context_ctor(ModeDecisionContext *ctx, EbColorFormat color_format,
-                                                      uint8_t sb_size, EncMode enc_mode, uint16_t max_block_cnt,
-                                                      uint32_t encoder_bit_depth,
-                                                      EbFifo  *mode_decision_configuration_input_fifo_ptr,
-                                                      EbFifo  *mode_decision_output_fifo_ptr,
-                                                      uint8_t enable_hbd_mode_decision, uint8_t cfg_palette);
+extern EbErrorType svt_aom_mode_decision_context_ctor(
+    ModeDecisionContext *ctx, EbColorFormat color_format, uint8_t sb_size, EncMode enc_mode, uint16_t max_block_cnt,
+    uint32_t encoder_bit_depth, EbFifo *mode_decision_configuration_input_fifo_ptr,
+    EbFifo *mode_decision_output_fifo_ptr, uint8_t enable_hbd_mode_decision, uint8_t cfg_palette, uint8_t seq_qp_mod);
 
 extern const EbAv1LambdaAssignFunc svt_aom_av1_lambda_assignment_function_table[4];
 
