@@ -23,7 +23,6 @@
 
 #include <assert.h>
 
-#define AV1_COST_PRECISION 0
 #define MV_COST_WEIGHT 108
 int svt_aom_get_reference_mode_context_new(const MacroBlockD *xd);
 int svt_av1_get_pred_context_uni_comp_ref_p(const MacroBlockD *xd);
@@ -113,8 +112,8 @@ void svt_av1_txb_init_levels_c(const TranLow *const coeff, const int32_t width,
 
 static int32_t av1_transform_type_rate_estimation(struct ModeDecisionContext *ctx, uint8_t allow_update_cdf,
                                                   FRAME_CONTEXT *fc, struct ModeDecisionCandidateBuffer *cand_bf,
-                                                  Bool is_inter, TxSize transform_size, TxType transform_type,
-                                                  Bool reduced_tx_set_used) {
+                                                  bool is_inter, TxSize transform_size, TxType transform_type,
+                                                  bool reduced_tx_set_used) {
     // const MbModeInfo *mbmi = &xd->mi[0]->mbmi;
     // const int32_t is_inter = is_inter_block(mbmi);
 
@@ -435,7 +434,7 @@ static INLINE int32_t av1_cost_coeffs_txb_loop_cost_eob(struct ModeDecisionConte
 uint64_t svt_av1_cost_coeffs_txb(struct ModeDecisionContext *ctx, uint8_t allow_update_cdf, FRAME_CONTEXT *ec_ctx,
                                  struct ModeDecisionCandidateBuffer *cand_bf, const TranLow *const qcoeff, uint16_t eob,
                                  PlaneType plane_type, TxSize transform_size, TxType transform_type,
-                                 int16_t txb_skip_ctx, int16_t dc_sign_ctx, Bool reduced_transform_set_flag)
+                                 int16_t txb_skip_ctx, int16_t dc_sign_ctx, bool reduced_transform_set_flag)
 
 {
     //Note: there is a different version of this function in AOM that seems to be efficient as its name is:
@@ -469,7 +468,7 @@ uint64_t svt_av1_cost_coeffs_txb(struct ModeDecisionContext *ctx, uint8_t allow_
                                 width,
                                 height,
                                 levels); // NM - Needs to be optimized - to be combined with the quantisation.
-    const Bool is_inter = is_inter_mode(cand_bf->cand->pred_mode);
+    const bool is_inter = is_inter_mode(cand_bf->cand->pred_mode);
     // Transform type bit estimation
     cost += plane_type > PLANE_TYPE_Y ? 0
                                       : av1_transform_type_rate_estimation(ctx,
@@ -634,8 +633,8 @@ uint64_t svt_aom_intra_fast_cost(PictureControlSet *pcs, struct ModeDecisionCont
         rate                      = mv_rate + ctx->md_rate_est_ctx->intrabc_fac_bits[cand->use_intrabc];
         cand_bf->fast_luma_rate   = rate;
         cand_bf->fast_chroma_rate = 0;
-        uint64_t luma_sad         = (LUMA_WEIGHT * luma_distortion) << AV1_COST_PRECISION;
-        uint64_t chromasad_       = chroma_distortion << AV1_COST_PRECISION;
+        uint64_t luma_sad         = luma_distortion;
+        uint64_t chromasad_       = chroma_distortion;
         uint64_t total_distortion = luma_sad + chromasad_;
 
         return (RDCOST(lambda, rate, total_distortion));
@@ -734,8 +733,8 @@ uint64_t svt_aom_intra_fast_cost(PictureControlSet *pcs, struct ModeDecisionCont
         // Keep the Fast Luma and Chroma rate for future use
         cand_bf->fast_luma_rate   = luma_rate;
         cand_bf->fast_chroma_rate = chroma_rate;
-        luma_sad                  = (LUMA_WEIGHT * luma_distortion) << AV1_COST_PRECISION;
-        chromasad_                = chroma_distortion << AV1_COST_PRECISION;
+        luma_sad                  = luma_distortion;
+        chromasad_                = chroma_distortion;
         total_distortion          = luma_sad + chromasad_;
 
         rate = luma_rate + chroma_rate;
@@ -753,7 +752,7 @@ static INLINE int svt_aom_has_uni_comp_refs(const MbModeInfo *mbmi) {
 
 // This function encodes the reference frame
 uint64_t estimate_ref_frame_type_bits(struct ModeDecisionContext *ctx, BlkStruct *blk_ptr, uint8_t ref_frame_type,
-                                      Bool is_compound) {
+                                      bool is_compound) {
     uint64_t ref_rate_bits = 0;
 
     // const MbModeInfo *const mbmi = &blk_ptr->av1xd->mi[0]->mbmi;
@@ -1113,8 +1112,8 @@ static uint64_t av1_inter_fast_cost_light(struct ModeDecisionContext *ctx, BlkSt
     // Keep the Fast Luma and Chroma rate for future use
     cand_bf->fast_luma_rate   = luma_rate;
     cand_bf->fast_chroma_rate = chroma_rate;
-    luma_sad                  = (LUMA_WEIGHT * luma_distortion) << AV1_COST_PRECISION;
-    chromasad_                = chroma_distortion << AV1_COST_PRECISION;
+    luma_sad                  = luma_distortion;
+    chromasad_                = chroma_distortion;
     total_distortion          = luma_sad + chromasad_;
     //if (blk_geom->has_uv == 0 && chromasad_ != 0)
     //    SVT_LOG("svt_aom_inter_fast_cost: Chroma error");
@@ -1292,7 +1291,7 @@ uint64_t svt_aom_inter_fast_cost(PictureControlSet *pcs, struct ModeDecisionCont
         /* Check if inter-intra is allowed for current block size / mode (even if the feature is off
         * for the current block, we still need to signal inter-intra off.
         */
-        svt_is_interintra_allowed(TRUE, blk_geom->bsize, cand->pred_mode, rf)) {
+        svt_is_interintra_allowed(true, blk_geom->bsize, cand->pred_mode, rf)) {
         const int interintra  = cand->is_interintra_used;
         const int bsize_group = size_group_lookup[blk_geom->bsize];
 
@@ -1312,7 +1311,7 @@ uint64_t svt_aom_inter_fast_cost(PictureControlSet *pcs, struct ModeDecisionCont
             }
         }
     }
-    Bool is_inter = inter_mode >= SINGLE_INTER_MODE_START && inter_mode < SINGLE_INTER_MODE_END;
+    bool is_inter = inter_mode >= SINGLE_INTER_MODE_START && inter_mode < SINGLE_INTER_MODE_END;
     if (is_inter && frm_hdr->is_motion_mode_switchable && rf[1] != INTRA_FRAME) {
         MotionMode motion_mode_rd           = cand->motion_mode;
         BlockSize  bsize                    = blk_geom->bsize;
@@ -1360,8 +1359,8 @@ uint64_t svt_aom_inter_fast_cost(PictureControlSet *pcs, struct ModeDecisionCont
     // Keep the Fast Luma and Chroma rate for future use
     cand_bf->fast_luma_rate   = luma_rate;
     cand_bf->fast_chroma_rate = chroma_rate;
-    luma_sad                  = (LUMA_WEIGHT * luma_distortion) << AV1_COST_PRECISION;
-    chromasad_                = chroma_distortion << AV1_COST_PRECISION;
+    luma_sad                  = luma_distortion;
+    chromasad_                = chroma_distortion;
     total_distortion          = luma_sad + chromasad_;
     if (blk_geom->has_uv == 0 && chromasad_ != 0)
         SVT_ERROR("svt_aom_inter_fast_cost: Chroma error");
@@ -1423,7 +1422,7 @@ EbErrorType svt_aom_txb_estimate_coeff_bits(struct ModeDecisionContext *ctx, uin
     int16_t  cr_txb_skip_context   = ctx->cr_txb_skip_context;
     int16_t  cr_dc_sign_context    = ctx->cr_dc_sign_context;
 
-    Bool reduced_transform_set_flag = frm_hdr->reduced_tx_set ? TRUE : FALSE;
+    bool reduced_transform_set_flag = frm_hdr->reduced_tx_set ? true : false;
 
     //Estimate the rate of the transform type and coefficient for Luma
 
@@ -1595,7 +1594,7 @@ void svt_aom_full_cost(PictureControlSet *pcs, ModeDecisionContext *ctx, struct
     uint64_t mode_cost            = RDCOST(lambda, mode_rate, mode_distortion);
 
     // If skip_mode is allowed for this candidate, check cost of skip mode compared to regular cost
-    if (cand_bf->cand->skip_mode_allowed == TRUE) {
+    if (cand_bf->cand->skip_mode_allowed == true) {
         const uint8_t skip_mode_ctx = ctx->skip_mode_ctx;
 
         // Skip mode cost
@@ -1607,14 +1606,14 @@ void svt_aom_full_cost(PictureControlSet *pcs, ModeDecisionContext *ctx, struct
             : 0;
         const uint64_t skip_mode_cost            = RDCOST(lambda, skip_mode_rate, skip_mode_distortion);
 
-        cand_bf->cand->skip_mode = FALSE;
+        cand_bf->cand->skip_mode = false;
         if (skip_mode_cost <= mode_cost) {
             // Update candidate cost
             mode_cost                = skip_mode_cost;
             mode_rate                = skip_mode_rate;
             mode_distortion          = skip_mode_distortion;
             mode_ssim_distortion     = skip_mode_ssim_distortion;
-            cand_bf->cand->skip_mode = TRUE;
+            cand_bf->cand->skip_mode = true;
 
             // Update signals to correspond to skip_mode values (no coeffs, etc.)
             cand_bf->block_has_coeff = 0;
@@ -1647,13 +1646,8 @@ void svt_aom_full_cost(PictureControlSet *pcs, ModeDecisionContext *ctx, struct
  * Coding Loop Context Generation
  ************************************************************/
 void svt_aom_coding_loop_context_generation(PictureControlSet *pcs, ModeDecisionContext *ctx) {
-    BlkStruct         *blk_ptr                       = ctx->blk_ptr;
-    uint32_t           blk_org_x                     = ctx->blk_org_x;
-    uint32_t           blk_org_y                     = ctx->blk_org_y;
-    NeighborArrayUnit *leaf_partition_na             = ctx->leaf_partition_na;
-    uint32_t           partition_left_neighbor_index = get_neighbor_array_unit_left_index(leaf_partition_na, blk_org_y);
-    uint32_t           partition_above_neighbor_index = get_neighbor_array_unit_top_index(leaf_partition_na, blk_org_x);
-    MacroBlockD       *xd                             = blk_ptr->av1xd;
+    BlkStruct   *blk_ptr = ctx->blk_ptr;
+    MacroBlockD *xd      = blk_ptr->av1xd;
     if (!ctx->shut_fast_rate) {
         if (pcs->slice_type == I_SLICE) {
             svt_aom_get_kf_y_mode_ctx(xd, &ctx->intra_luma_top_ctx, &ctx->intra_luma_left_ctx);
@@ -1661,19 +1655,6 @@ void svt_aom_coding_loop_context_generation(PictureControlSet *pcs, ModeDecision
         ctx->is_inter_ctx  = svt_av1_get_intra_inter_context(xd);
         ctx->skip_mode_ctx = av1_get_skip_mode_context(xd);
     }
-    // Generate Partition context
-    blk_ptr->above_neighbor_partition =
-        (((PartitionContext *)leaf_partition_na->top_array)[partition_above_neighbor_index].above ==
-         (char)INVALID_NEIGHBOR_DATA)
-        ? 0
-        : ((PartitionContext *)leaf_partition_na->top_array)[partition_above_neighbor_index].above;
-
-    blk_ptr->left_neighbor_partition =
-        (((PartitionContext *)leaf_partition_na->left_array)[partition_left_neighbor_index].left ==
-         (char)INVALID_NEIGHBOR_DATA)
-        ? 0
-        : ((PartitionContext *)leaf_partition_na->left_array)[partition_left_neighbor_index].left;
-
     // Collect Neighbor ref cout
     if (pcs->slice_type != I_SLICE || pcs->ppcs->frm_hdr.allow_intrabc)
         svt_aom_collect_neighbors_ref_counts_new(blk_ptr->av1xd);
@@ -1905,13 +1886,13 @@ static uint64_t cost_selected_tx_size(const MacroBlockD *xd, MdRateEstimationCon
 
 /* Get the TXS rate and update the txfm context.  If allow_update_cdf is true, the TX size CDFs will
 be updated. */
-uint64_t svt_aom_tx_size_bits(MdRateEstimationContext *md_rate_est_ctx, MacroBlockD *xd, const MbModeInfo *mbmi,
-                              TxSize tx_size, TxMode tx_mode, BlockSize bsize, uint8_t skip, FRAME_CONTEXT *ec_ctx,
-                              uint8_t allow_update_cdf) {
+uint64_t svt_aom_tx_size_bits(PictureControlSet *pcs, uint8_t segment_id, MdRateEstimationContext *md_rate_est_ctx,
+                              MacroBlockD *xd, const MbModeInfo *mbmi, TxSize tx_size, TxMode tx_mode, BlockSize bsize,
+                              uint8_t skip, FRAME_CONTEXT *ec_ctx, uint8_t allow_update_cdf) {
     uint64_t bits        = 0;
     int      is_inter_tx = is_inter_block(&mbmi->block_mi);
-    if (tx_mode == TX_MODE_SELECT && block_signals_txsize(bsize) &&
-        !(is_inter_tx && skip) /*&& !xd->lossless[segment_id]*/) {
+    if (tx_mode == TX_MODE_SELECT && block_signals_txsize(bsize) && !(is_inter_tx && skip) &&
+        !svt_av1_is_lossless_segment(pcs, segment_id)) {
         if (is_inter_tx) { // This implies skip flag is 0.
             const TxSize max_tx_size = get_vartx_max_txsize(/*xd,*/ bsize, 0);
             const int    txbh        = tx_size_high_unit[max_tx_size];
@@ -1937,7 +1918,7 @@ uint64_t svt_aom_tx_size_bits(MdRateEstimationContext *md_rate_est_ctx, MacroBlo
 /* Get the TXS rate.  A dummy txfm context array will be used, so context updates will not be saved for
 future blocks. */
 uint64_t svt_aom_get_tx_size_bits(ModeDecisionCandidateBuffer *candidateBuffer, ModeDecisionContext *ctx,
-                                  PictureControlSet *pcs, uint8_t tx_depth, Bool block_has_coeff) {
+                                  PictureControlSet *pcs, uint8_t tx_depth, bool block_has_coeff) {
     NeighborArrayUnit *txfm_context_array      = ctx->txfm_context_array;
     uint32_t           txfm_context_left_index = get_neighbor_array_unit_left_index(txfm_context_array, ctx->blk_org_y);
     uint32_t           txfm_context_above_index = get_neighbor_array_unit_top_index(txfm_context_array, ctx->blk_org_x);
@@ -1961,8 +1942,17 @@ uint64_t svt_aom_get_tx_size_bits(ModeDecisionCandidateBuffer *candidateBuffer,
     mbmi->block_mi.ref_frame[0] = candidateBuffer->cand->ref_frame_type;
     mbmi->block_mi.tx_depth     = tx_depth;
 
-    const uint64_t bits = svt_aom_tx_size_bits(
-        ctx->md_rate_est_ctx, xd, mbmi, ctx->blk_geom->txsize[tx_depth], tx_mode, bsize, !block_has_coeff, NULL, 0);
+    const uint64_t bits = svt_aom_tx_size_bits(pcs,
+                                               ctx->blk_ptr->segment_id,
+                                               ctx->md_rate_est_ctx,
+                                               xd,
+                                               mbmi,
+                                               ctx->blk_geom->txsize[tx_depth],
+                                               tx_mode,
+                                               bsize,
+                                               !block_has_coeff,
+                                               NULL,
+                                               0);
     return bits;
 }
 
@@ -1977,7 +1967,7 @@ uint64_t svt_aom_partition_rate_cost(PictureParentControlSet *pcs, ModeDecisionC
     const BlockSize  bsize    = blk_geom->bsize;
     assert(mi_size_wide_log2[bsize] == mi_size_high_log2[bsize]);
     assert(bsize < BlockSizeS_ALL);
-    const Bool is_partition_point = (bsize >= BLOCK_8X8);
+    const bool is_partition_point = (bsize >= BLOCK_8X8);
 
     if (!is_partition_point) {
         return 0;
@@ -1988,10 +1978,13 @@ uint64_t svt_aom_partition_rate_cost(PictureParentControlSet *pcs, ModeDecisionC
     const int hbs       = (mi_size_wide[bsize] << 2) >> 1;
     const int has_rows  = (blk_org_y + hbs) < pcs->aligned_height;
     const int has_cols  = (blk_org_x + hbs) < pcs->aligned_width;
-
+    // Don't consider blocks outside the picture
+    if (blk_org_y >= pcs->aligned_height || blk_org_x >= pcs->aligned_width)
+        return 0;
     if (!has_rows && !has_cols) {
         return 0;
     }
+
     const PartitionContextType left_ctx  = ctx->md_blk_arr_nsq[blk_mds_idx].left_neighbor_partition ==
             (char)(INVALID_NEIGHBOR_DATA)
          ? 0
@@ -2004,22 +1997,34 @@ uint64_t svt_aom_partition_rate_cost(PictureParentControlSet *pcs, ModeDecisionC
     assert(bsl >= 0);
 
     const int      above = (above_ctx >> bsl) & 1, left = (left_ctx >> bsl) & 1;
-    const int      partitio_ploffset = use_accurate_part_ctx ? PARTITION_PLOFFSET : 0;
-    const uint32_t context_index     = (left * 2 + above) + bsl * partitio_ploffset;
+    const uint32_t context_index = (left * 2 + above) + bsl * PARTITION_PLOFFSET;
 
     uint64_t split_rate = 0;
 
     if (has_rows && has_cols) {
         split_rate = (uint64_t)md_rate_est_ctx->partition_fac_bits[context_index][p];
     } else if (!has_rows && has_cols) {
+        // 8x8 blocks will not use the split_or_horz or the split_or_vert paritition CDFs, per
+        // section 8.3.2 of the AV1 spec (Cdf selection process).  Therefore, only update partition ctx 4+,
+        // which corresponds to the paritition CDFs for 16x16 and larger blocks
+        assert(bsize != BLOCK_8X8);
         split_rate = bsize == BLOCK_128X128
             ? (uint64_t)md_rate_est_ctx->partition_vert_alike_128x128_fac_bits[context_index][p == PARTITION_SPLIT]
             : (uint64_t)md_rate_est_ctx->partition_vert_alike_fac_bits[context_index][p == PARTITION_SPLIT];
     } else {
+        // 8x8 blocks will not use the split_or_horz or the split_or_vert paritition CDFs, per
+        // section 8.3.2 of the AV1 spec (Cdf selection process).  Therefore, only update partition ctx 4+,
+        // which corresponds to the paritition CDFs for 16x16 and larger blocks
+        assert(bsize != BLOCK_8X8);
         split_rate = bsize == BLOCK_128X128
             ? (uint64_t)md_rate_est_ctx->partition_horz_alike_128x128_fac_bits[context_index][p == PARTITION_SPLIT]
             : (uint64_t)md_rate_est_ctx->partition_horz_alike_fac_bits[context_index][p == PARTITION_SPLIT];
     }
 
+    // If not using accurate partition rate, bias against splitting by increasing the rate of SPLIT partition
+    if (!use_accurate_part_ctx && p == PARTITION_SPLIT) {
+        split_rate *= 2;
+    }
+
     return (RDCOST(lambda, split_rate, 0));
 }
