@@ -14,6 +14,7 @@
 #include "gtest/gtest.h"
 
 #include "aom_dsp_rtcd.h"
+#include "md_config_process.h"
 #include "random.h"
 #include "util.h"
 
@@ -26,11 +27,6 @@
 namespace {
 using svt_av1_test_tool::SVTRandom;
 
-extern "C" void svt_av1_build_quantizer(
-    EbBitDepth bit_depth, int32_t y_dc_delta_q, int32_t u_dc_delta_q,
-    int32_t u_ac_delta_q, int32_t v_dc_delta_q, int32_t v_ac_delta_q,
-    Quants *const quants, Dequants *const deq);
-
 #define QUAN_PARAM_LIST                                                      \
     const TranLow *coeff_ptr, intptr_t n_coeffs, const int16_t *zbin_ptr,    \
         const int16_t *round_ptr, const int16_t *quant_ptr,                  \
@@ -99,8 +95,13 @@ class QuantizeTest : public ::testing::TestWithParam<ParamType> {
     }
 
     void InitQuantizer() {
+        PictureParentControlSet pcs;
+        pcs.scs = (SequenceControlSet *)malloc(sizeof(SequenceControlSet));
+        pcs.frm_hdr.quantization_params.base_q_idx = 0;
+        pcs.scs->static_config.sharpness = 0;
+        PictureParentControlSet *pcs_ptr = &pcs;
         svt_av1_build_quantizer(
-            bd_, 0, 0, 0, 0, 0, &qtab_->quant, &qtab_->dequant);
+            pcs_ptr, bd_, 0, 0, 0, 0, 0, &qtab_->quant, &qtab_->dequant);
     }
 
     virtual void QuantizeRun(bool is_loop, int q = 0, int test_num = 1) = 0;
@@ -467,6 +468,7 @@ class QuantizeHbdTest : public QuantizeTest<QuantizeHbdParam, QuantizeHbdFunc> {
         }
     }
 };
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(QuantizeHbdTest);
 
 TEST_P(QuantizeHbdTest, ZeroInput) {
     FillCoeffZero();
@@ -727,6 +729,7 @@ class QuantizeQmTest : public QuantizeTest<QuantizeQmParam, QuantizeQmFunc> {
     const QmVal *qmatrix_[NUM_QM_LEVELS][3][TX_SIZES_ALL];
     int qm_level_;
 };
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(QuantizeQmTest);
 
 TEST_P(QuantizeQmTest, ZeroInput) {
     FillCoeffZero();
@@ -761,6 +764,7 @@ TEST_P(QuantizeQmTest, CoeffHalfDequant) {
 }
 
 using QuantizeQmHbdTest = QuantizeQmTest;
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(QuantizeQmHbdTest);
 
 TEST_P(QuantizeQmHbdTest, ZeroInput) {
     FillCoeffZero();
@@ -796,6 +800,8 @@ TEST_P(QuantizeQmHbdTest, CoeffHalfDequant) {
 
 using std::make_tuple;
 
+#ifdef ARCH_X86_64
+
 #if HAS_AVX2
 const QuantizeParam kQParamArrayAvx2[] = {
     make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_sse4_1,
@@ -968,36 +974,132 @@ INSTANTIATE_TEST_SUITE_P(AVX2, QuantizeQmTest,
                          ::testing::ValuesIn(kQmParamArrayAvx2));
 INSTANTIATE_TEST_SUITE_P(AVX2, QuantizeQmHbdTest,
                          ::testing::ValuesIn(kQmParamHbdArrayAvx2));
+#endif  // HAS_AVX2
+#endif  // ARCH_X86_64
 
-TEST(ComputeCulLevel, avx2) {
-    SVTRandom rnd(0, (1 << 10) - 1);
-    const int max_size = 100 * 100;
-    // scan[] is a set of indexes for quant_coeff[]
-    int16_t scan[max_size];
-    int32_t quant_coeff[max_size];
-    uint16_t eob_ref, eob_mod;
+#ifdef ARCH_AARCH64
+const QuantizeParam kQParamArrayNeon[] = {
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X16), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_neon,
+               static_cast<TxSize>(TX_4X16), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X4), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_neon,
+               static_cast<TxSize>(TX_32X8), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_c, &svt_av1_quantize_fp_neon,
+               static_cast<TxSize>(TX_8X32), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_32x32_c, &svt_av1_quantize_fp_32x32_neon,
+               static_cast<TxSize>(TX_32X32), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_32x32_c, &svt_av1_quantize_fp_32x32_neon,
+               static_cast<TxSize>(TX_16X64), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_32x32_c, &svt_av1_quantize_fp_32x32_neon,
+               static_cast<TxSize>(TX_64X16), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_quantize_fp_64x64_c, &svt_av1_quantize_fp_64x64_neon,
+               static_cast<TxSize>(TX_64X64), TYPE_FP, EB_EIGHT_BIT)};
 
-    scan[0] = 0;
+INSTANTIATE_TEST_SUITE_P(NEON, QuantizeLbdTest,
+                         ::testing::ValuesIn(kQParamArrayNeon));
 
-    for (int test = 0; test < 1000; test++) {
-        // Every 50 iteration randomize buffers
-        if (!(test % 50))
-            for (uint32_t i = 0; i < max_size; i++) {
-                quant_coeff[i] = rnd.random();
-                if (i != 0)
-                    scan[i] = rnd.random() % max_size;
-            }
+const QuantizeHbdParam kQHbdParamArrayNeon[] = {
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X16), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_4X16), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X4), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_32X8), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_8X32), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_64X64), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X16), TYPE_FP, EB_TEN_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_4X16), TYPE_FP, EB_TEN_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X4), TYPE_FP, EB_TEN_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_32X8), TYPE_FP, EB_TEN_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_8X32), TYPE_FP, EB_TEN_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_64X64), TYPE_FP, EB_EIGHT_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X16), TYPE_FP, EB_TWELVE_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_4X16), TYPE_FP, EB_TWELVE_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_16X4), TYPE_FP, EB_TWELVE_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_32X8), TYPE_FP, EB_TWELVE_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_8X32), TYPE_FP, EB_TWELVE_BIT),
+    make_tuple(&svt_av1_highbd_quantize_fp_c, &svt_av1_highbd_quantize_fp_neon,
+               static_cast<TxSize>(TX_64X64), TYPE_FP, EB_TWELVE_BIT)};
 
-        eob_ref = eob_mod = rnd.random() % max_size;
+INSTANTIATE_TEST_SUITE_P(NEON, QuantizeHbdTest,
+                         ::testing::ValuesIn(kQHbdParamArrayNeon));
+#endif  // ARCH_AARCH64
 
-        int32_t ref = svt_av1_compute_cul_level_c(scan, quant_coeff, &eob_ref);
-        int32_t mod =
-            svt_av1_compute_cul_level_avx2(scan, quant_coeff, &eob_mod);
+using ComputeCulLevelFunc = uint8_t (*)(const int16_t *const scan,
+                                        const int32_t *const quant_coeff,
+                                        uint16_t *eob);
 
-        EXPECT_TRUE(ref == mod);
-        EXPECT_TRUE(eob_ref == eob_mod);
+class ComputeCulLevelTest
+    : public ::testing::TestWithParam<ComputeCulLevelFunc> {
+  public:
+    ComputeCulLevelTest() : test_func_(GetParam()) {
     }
+
+    void test_match() {
+        SVTRandom rnd(0, (1 << 10) - 1);
+        SVTRandom quant_rnd(-10, 10);
+        const int max_size = 50;
+        // scan[] is a set of indexes for quant_coeff[]
+        int16_t scan[max_size];
+        int32_t quant_coeff[max_size];
+        uint16_t eob_ref, eob_test;
+
+        scan[0] = 0;
+
+        for (int test = 0; test < 1000; test++) {
+            // Every 50 iteration randomize buffers
+            if (!(test % 50))
+                for (uint32_t i = 0; i < max_size; i++) {
+                    quant_coeff[i] = quant_rnd.random();
+                    if (i != 0)
+                        scan[i] = rnd.random() % max_size;
+                }
+
+            eob_ref = eob_test = rnd.random() % max_size;
+
+            int32_t ref_res =
+                svt_av1_compute_cul_level_c(scan, quant_coeff, &eob_ref);
+
+            int32_t test_res = test_func_(scan, quant_coeff, &eob_test);
+
+            EXPECT_EQ(ref_res, test_res);
+            EXPECT_EQ(eob_ref, eob_test);
+        }
+    }
+
+  private:
+    ComputeCulLevelFunc test_func_;
+};
+
+TEST_P(ComputeCulLevelTest, test_match) {
+    test_match();
 }
 
-#endif  // HAS_AVX2
+#ifdef ARCH_X86_64
+INSTANTIATE_TEST_SUITE_P(AVX2, ComputeCulLevelTest,
+                         ::testing::Values(svt_av1_compute_cul_level_avx2));
+#endif  // ARCH_X86_64
+
+#ifdef ARCH_AARCH64
+INSTANTIATE_TEST_SUITE_P(NEON, ComputeCulLevelTest,
+                         ::testing::Values(svt_av1_compute_cul_level_neon));
+#endif  // ARCH_AARCH64
 }  // namespace
